@17140@
@13830@
@13245@
@13635@
@13440@
@13245@

Modified by Robert H”hne to be used for RHIDE.

@13245@
@13245@
@14024@

@16394@
@15096@

@19872@
@23372@

@21035@

@08331@

@08154@
@05453@
@05061@
@05062@
@05064@
@05063@
@06020@

@10911@

@10891@
@10888@

@10953@
@10889@
    static void GetCursor(int &x, int &y);
    static void SetCursor(int x, int y);
@10954@
@10890@
    static void SetPage(uchar);
    static uchar GetPage() { return Page; }

@20938@

@06634@
@06633@
@12575@

@20890@

@10971@
@10970@

    static ushort * equipment;
    static uchar * crtInfo;
    static uchar * crtRows;
    static uchar Page;

@24833@

@20048@
@23372@

@21035@

@07012@
@12599@

@10959@
@10913@

@10902@
@10901@
@10899@
@10869@
@10868@
@10562@
@10552@
    static long screenBuffer;
@10880@

@10952@
@10885@

@10966@
@10944@
    static void GetCursor(int &x, int &y);
    static void SetCursor(int x, int y);
@10954@
@10890@
@10891@
@10888@

    static ushort getCharacter(unsigned offset);
    static void getCharacter(unsigned offset,ushort *buf,unsigned count);
    static void setCharacter(unsigned offset,ushort value);
    static void setCharacter(unsigned offset,ushort *values,unsigned count);

@24833@

@15680@

